﻿# include <Siv3D.hpp>
using namespace System;
using namespace Scene;
using namespace Palette;

using App = SceneManager<String>;

int WINDOW_WIDTH = 800;
int WINDOW_HEIGHT = 600;
int score;

class Object {
public:
	int x;
	int y;
	int width;
	int height;
	Color color;
	int type1;
	int type2;
	RectF rect;
	Circle circle;

	void draw() const {
		if (type1 == 0) {
			Circle{ x, y, width / 2 }.draw(color);
		}
		if (type1 == 1) {
			RectF{ x - width / 2, y - height / 2, width, height }.draw(color);
		}
	}

	void draw_rect() const {
		this->rect.draw(ColorF(color, 0));
	}

	void draw_circle() const {
		this->circle.draw(ColorF(color, 0));
	}

	void move(int dx, int dy) {
		x += dx / 2;
		y += dy / 2;

		if (type2 == 0) {
			if (x - width / 2 < WINDOW_WIDTH / 2 + 10) {
				x = WINDOW_WIDTH / 2 + 10 + width / 2;
			}
			if (x + width / 2 > WINDOW_WIDTH) {
				x = WINDOW_WIDTH - width / 2;
			}
			if (y - height / 2 < 0) {
				y = height / 2;
			}
			if (y + height / 2 > WINDOW_HEIGHT) {
				y = WINDOW_HEIGHT - height / 2;
			}
		}
		else {
			if (x - width / 2 < 0) {
				x = width / 2;
			}
			if (x + width / 2 > WINDOW_WIDTH / 2) {
				x = WINDOW_WIDTH / 2 - width / 2;
			}
			if (y - height / 2 < 0) {
				y = height / 2;
			}
			if (y + height / 2 > WINDOW_HEIGHT) {
				y = WINDOW_HEIGHT - height / 2;
			}
		}

		// 当たり判定用の四角形と円を更新
		this->rect = RectF{ x - width / 2, y - height / 2, width, height };
		this->circle = Circle{ x, y, width / 2 };
	}

	bool checkCollision(const Circle& bullet) const {
		return bullet.intersects(rect);
	}
};

class Title : public App::Scene {
public:
	Title(const InitData& init)
		: IScene{ init } {}

	void update() override {
		if (KeyS.down()) {
			changeScene(U"Game");
		}
	}

	void draw() const override {
		Scene::SetBackground(ColorF{ Green });
		FontAsset(U"BigFont")(U"脳トレシューティング").drawAt(400, 100);
		FontAsset(U"TitleFont")(U"操作説明").drawAt(400, 200);
		FontAsset(U"TitleFont")(U"青　WASD").drawAt(200, 300);
		FontAsset(U"TitleFont")(U"赤　矢印キー").drawAt(600, 300);
		FontAsset(U"TitleFont")(U"スタート　S").drawAt(400, 400);
		score = 0;
	}
};

class Result : public App::Scene {
public:
	Result(const InitData& init)
		:IScene{ init } {}
	void update() override {
		if (KeySpace.down()) {
			changeScene(U"Game");
		}
		if (KeyE.down()) {
			changeScene(U"Title");
		}
	}
	void draw() const override {
		SetBackground(ColorF(Black));
		FontAsset(U"BigFont")(U"Game Over").drawAt(WINDOW_WIDTH / 2, WINDOW_HEIGHT / 2 - 100);
		if (score < 500) {
			FontAsset(U"TitleFont")(U"わざと当たってる？").drawAt(WINDOW_WIDTH / 2, WINDOW_HEIGHT / 2 - 50);
		}
		else if (score < 1000) {
			FontAsset(U"TitleFont")(U"シューティング初心者").drawAt(WINDOW_WIDTH / 2, WINDOW_HEIGHT / 2 - 50);
		}
		else if (score < 2000) {
			FontAsset(U"TitleFont")(U"避けるのは上手い").drawAt(WINDOW_WIDTH / 2, WINDOW_HEIGHT / 2 - 50);
		}
		else if (score < 3000) {
			FontAsset(U"TitleFont")(U"運がよかったね").drawAt(WINDOW_WIDTH / 2, WINDOW_HEIGHT / 2 - 50);
		}
		else if (score < 4000) {
			FontAsset(U"TitleFont")(U"まだセリフがあるよ？").drawAt(WINDOW_WIDTH / 2, WINDOW_HEIGHT / 2 - 50);
		}
		else {
			FontAsset(U"TitleFont")(U"本当にありがとうございました").drawAt(WINDOW_WIDTH / 2, WINDOW_HEIGHT / 2 - 50);
		}
		FontAsset(U"TitleFont")(U"スコア：{}"_fmt(score)).drawAt(WINDOW_WIDTH / 2, WINDOW_HEIGHT / 2);
		FontAsset(U"TitleFont")(U"コンティニュー　スペースキー").drawAt(WINDOW_WIDTH / 2, WINDOW_HEIGHT / 2 + 50);
		FontAsset(U"TitleFont")(U"終わる E").drawAt(WINDOW_WIDTH / 2, WINDOW_HEIGHT / 2 + 100);
	}
};

class Game : public App::Scene {
private:
	Object object;
	Object object1;
	Object bullet;
	Object bullet1;
	Object bullet2;
	Array<Object> bullets;
	Array<Object> bullets1;
	Array<Object> bullets2;
	RectF wall;


	int count;
	bool running = true;

public:
	Game(const InitData& init)
		: IScene{ init }, count(1000) {
		Window::Resize(WINDOW_WIDTH, WINDOW_HEIGHT);
		SetBackground(ColorF(0, 0, 0));
		score = 0;

		object.x = WINDOW_WIDTH / 2 + 100;
		object.y = WINDOW_HEIGHT / 2 + 100;
		object.width = 30;
		object.height = 30;
		object.color = Red;
		object.rect = { object.x,object.y,object.width,object.height };
		object.type1 = 1;
		object.type2 = 0;

		object1.x = WINDOW_WIDTH / 2 - 100;
		object1.y = WINDOW_HEIGHT / 2 + 100;
		object1.width = 30;
		object1.height = 30;
		object1.color = Blue;
		object1.rect = { object1.x,object1.y,object1.width,object1.height };
		object1.type1 = 1;
		object1.type2 = 1;

		bullet.width = 30;
		bullet.height = 30;
		bullet.color = Yellow;
		bullet.type1 = 0;
		bullet.circle;

		bullet1.width = 30;
		bullet1.height = 30;
		bullet1.color = Green;
		bullet1.type1 = 0;
		bullet1.circle;

		bullet2.width = 30;
		bullet2.height = 30;
		bullet2.color = Orange;
		bullet2.type1 = 0;
		bullet2.circle;

		wall = { WINDOW_WIDTH / 2, 0, 10, WINDOW_HEIGHT };
	}

	void update() override {
		ClearPrint();
		const double deltaTime = Scene::DeltaTime();

		object.move(0, 0);
		object1.move(0, 0);

		if (running) {
			score += deltaTime * 80;
		}

		// 弾を生成して配列に追加
		if (Random(100) < 5 && running) {
			Object newBullet = bullet;
			newBullet.x = Random(0, WINDOW_WIDTH - bullet.width);
			newBullet.y = 0;
			bullets.push_back(newBullet);
			newBullet.circle = { newBullet.x,newBullet.y,15 };
		}

		if (count > 0) {
			count -= deltaTime;
		}

		if (count <= 0) {
			count = 0;
			if (Random(100) < 1 && running) {
				Object newBullet1 = bullet1;
				newBullet1.x = 800;
				newBullet1.y = object.y;
				bullets1.push_back(newBullet1);
				newBullet1.circle = { newBullet1.x,newBullet1.y,15 };
			}

			if (Random(100) < 1 && running) {
				Object newBullet2 = bullet2;
				newBullet2.x = 0;
				newBullet2.y = object1.y;
				bullets2.push_back(newBullet2);
				newBullet2.circle = { newBullet2.x,newBullet2.y,15 };
			}
		}

		for (auto& bullet : bullets) {
			bullet.y += deltaTime * 150;
		}

		for (auto& bullet1 : bullets1) {
			if (running) {
				bullet1.x -= deltaTime * 150;
			}
			bullet1.draw_circle();
			bullet1.draw();

			double distanceToCharacter = Vec2(object.x, object.y).distanceFrom(Vec2(bullet1.x, bullet1.y));
			double distanceToCharacter1 = Vec2(object1.x, object1.y).distanceFrom(Vec2(bullet1.x, bullet1.y));
			double characterRadius = object.width / 2;
			double characterRadius1 = object1.width / 2;
			double bulletRadius1 = bullet1.width / 2;

			if (distanceToCharacter < characterRadius + bulletRadius1 || distanceToCharacter1 < characterRadius1 + bulletRadius1) {
				running = false;
			}
		}

		for (auto& bullet2 : bullets2) {
			if (running) {
				bullet2.x += deltaTime * 150;
			}
			bullet2.draw_circle();
			bullet2.draw();

			double distanceToCharacter = Vec2(object.x, object.y).distanceFrom(Vec2(bullet2.x, bullet2.y));
			double distanceToCharacter1 = Vec2(object1.x, object1.y).distanceFrom(Vec2(bullet2.x, bullet2.y));
			double characterRadius = object.width / 2;
			double characterRadius1 = object1.width / 2;
			double bulletRadius2 = bullet2.width / 2;

			if (distanceToCharacter < characterRadius + bulletRadius2 || distanceToCharacter1 < characterRadius1 + bulletRadius2) {
				running = false;
			}
		}

		bullets.remove_if([](const Object& bullet) {
			return bullet.y > WINDOW_HEIGHT;
		});

		bullets1.remove_if([](const Object& bullet) {
			return bullet.x < WINDOW_WIDTH / 2;
		});

		bullets2.remove_if([](const Object& bullet) {
			return bullet.x > WINDOW_WIDTH / 2;
		});

		wall.draw(White);
		object.draw_rect();
		object.draw();
		object1.draw_rect();
		object1.draw();

		for (auto& bullet : bullets) {
			bullet.draw_circle();
			bullet.draw();

			double distanceToCharacter = Vec2(object.x, object.y).distanceFrom(Vec2(bullet.x, bullet.y));
			double distanceToCharacter1 = Vec2(object1.x, object1.y).distanceFrom(Vec2(bullet.x, bullet.y));
			double characterRadius = object.width / 2;
			double characterRadius1 = object1.width / 2;
			double bulletRadius = bullet.width / 2;

			if (distanceToCharacter < characterRadius + bulletRadius || distanceToCharacter1 < characterRadius1 + bulletRadius) {
				running = false;
			}
		}

		if (object.type2 == 0 && running) {
			if (KeyRight.pressed()) {
				object.move(10, 0);
			}
			if (KeyLeft.pressed()) {
				object.move(-10, 0);
			}
			if (KeyUp.pressed()) {
				object.move(0, -10);
			}
			if (KeyDown.pressed()) {
				object.move(0, 10);
			}
		}

		if (object1.type2 == 1 && running) {
			if (KeyD.pressed()) {
				object1.move(10, 0);
			}
			if (KeyA.pressed()) {
				object1.move(-10, 0);
			}
			if (KeyW.pressed()) {
				object1.move(0, -10);
			}
			if (KeyS.pressed()) {
				object1.move(0, 10);
			}
		}
		if (running) {
			Print << U"スコア：{}"_fmt(score);
		}
		if (not running) {
			changeScene(U"Result");

		}
	}
};

void Main() {

	FontAsset::Register(U"TitleFont", 50, Typeface::Heavy);
	FontAsset::Register(U"BigFont", 60, Typeface::Heavy);
	App manager;
	manager.add<Title>(U"Title");
	manager.add<Game>(U"Game");
	manager.add<Result>(U"Result");
	while (System::Update()) {
		if (not manager.update()) {
			break;
		}
	}
}
